# Copyright 2022 The envd Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Config functions

::: tip
Note that the documentation is automatically generated from [envd/api](https://github.com/tensorchord/envd/tree/main/envd/api) folder
in [tensorchord/envd](https://github.com/tensorchord/envd/tree/main/envd/api) repo.
Please update the python file there instead of directly editing file inside envd-docs repo.
:::
"""

from typing import Optional, List


def apt_source(source: Optional[str]):
    """Configure apt sources

    Example usage:
    ```python
    apt_source(source='''
        deb https://mirror.sjtu.edu.cn/ubuntu focal main restricted
        deb https://mirror.sjtu.edu.cn/ubuntu focal-updates main restricted
        deb https://mirror.sjtu.edu.cn/ubuntu focal universe
        deb https://mirror.sjtu.edu.cn/ubuntu focal-updates universe
        deb https://mirror.sjtu.edu.cn/ubuntu focal multiverse
        deb https://mirror.sjtu.edu.cn/ubuntu focal-updates multiverse
        deb https://mirror.sjtu.edu.cn/ubuntu focal-backports main restricted universe multiverse
        deb http://archive.canonical.com/ubuntu focal partner
        deb https://mirror.sjtu.edu.cn/ubuntu focal-security main restricted universe multiverse
    ''')
    ```

    Args:
        source (str, optional): The apt source configuration
    """


def jupyter(token: str, port: int):
    """Configure jupyter notebook configuration

    Args:
        token (str): Token for access authentication
        port (int): Port to serve jupyter notebook
    """


def pip_index(url: str, extra_url: str = "", trust: bool = False):
    """Configure pypi index mirror

    Args:
        url (str): PyPI index URL (i.e. https://mirror.sjtu.edu.cn/pypi/web/simple)
        extra_url (str): PyPI extra index URL. `url` and `extra_url` will be
            treated equally, see https://github.com/pypa/pip/issues/8606
        trust (bool): trust the provided index
    """


def conda_channel(channel: str):
    """Configure conda channel mirror

    Example usage:
    ```python
    config.conda_channel(channel='''
    channels:
        - defaults
    show_channel_urls: true
    default_channels:
        - https://mirrors.tuna.tsinghua.edu.cn/anaconda/pkgs/main
        - https://mirrors.tuna.tsinghua.edu.cn/anaconda/pkgs/r
        - https://mirrors.tuna.tsinghua.edu.cn/anaconda/pkgs/msys2
    custom_channels:
        conda-forge: https://mirrors.tuna.tsinghua.edu.cn/anaconda/cloud
    ''')
    ```

    Args:
        channel (str): Basically the same with file content of an usual .condarc
    """


def entrypoint(args: List[str]):
    """Configure entrypoint for custom base image

    Example usage:
    ```python
    config.entrypoint(["date", "-u"])
    ```

    Args:
        args (List[str]): list of arguments to run
    """


def gpu(count: int):
    """Configure the number of GPUs required

    Example usage:
    ```python
    config.gpu(count=2)
    ```

    Args:
        count (int): number of GPUs
    """


def shm_size(size: int):
    """Configure the shared memory size (megabyte) of docker containers

    Example usage:
    ```python
    config.shm_size(size=1024)
    ```

    Args:
        size (int): the shared memory size (megabyte) of docker containers
    """


def cran_mirror(url: str):
    """Configure the mirror URL, default is https://cran.rstudio.com

    Args:
        url (str): mirror URL
    """


def julia_pkg_server(url: str):
    """Configure the package server for Julia.
    Since Julia 1.5, https://pkg.julialang.org is the default pkg server.

    Args:
        url (str): Julia pkg server URL
    """


def rstudio_server():
    """
    Enable the RStudio Server (only work for `base(os="ubuntu20.04", language="r")`)
    """


def repo(url: str, description: str):
    """Setup repo related information. Will save to the image labels.

    Args:
        url (str): repo URL
        description (str): repo description
    """



# Copyright 2022 The envd Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""IO functions

::: tip
Note that the documentation is automatically generated from [envd/api](https://github.com/tensorchord/envd/tree/main/envd/api) folder
in [tensorchord/envd](https://github.com/tensorchord/envd/tree/main/envd/api) repo.
Please update the python file there instead of directly editing file inside envd-docs repo.
:::

::: warning
Enable v1 by adding `# syntax=v1` to the 1st line of your envd file.

v1 is experimental and may change in the future. Make sure to freeze the envd version for online CI/CD.
:::
"""

from typing import Optional


def copy(source: str, target: str, image: Optional[str]):
    """Copy from host path to container path (build time)

    Args:
        source (str): source path in the host machine or in the ``image``
        target (str): destination path in the envd container
        image(Optional[str]): image name, if not specified, will use the host

    Examples:
    ```python
    # copy from host to container
    io.copy(source='main.py', target='/home/envd/')
    # copy from image to container
    io.copy(source='/bin/micromamba', target='/usr/local/bin/micromamba', image='mambaorg/micromamba:1.0.0')
    ```
    """


def http(url: str, checksum: Optional[str], filename: Optional[str]):
    """Download file with HTTP to `/home/envd/extra_source`

    Args:
        url (str): URL
        checksum (Optional[str]): checksum for the downloaded file
        filename (Optional[str]): rewrite the filename
    """



# Copyright 2022 The envd Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Runtime functions

::: tip
Note that the documentation is automatically generated from [envd/api](https://github.com/tensorchord/envd/tree/main/envd/api) folder
in [tensorchord/envd](https://github.com/tensorchord/envd/tree/main/envd/api) repo.
Please update the python file there instead of directly editing file inside envd-docs repo.
:::

::: warning
Enable v1 by adding `# syntax=v1` to the 1st line of your envd file.

v1 is experimental and may change in the future. Make sure to freeze the envd version for online CI/CD.
:::
"""

from typing import List, Optional, Dict


def command(commands: Dict[str, str]):
    """Execute commands during runtime

    Args:
        commands (Dict[str, str]): map name to command, similar to Makefile

    Example usage:
    ```python
    runtime.command(commands={
        "train": "python train.py --epoch 20 --notify me@tensorchord.ai",
        "run": "python server.py --batch 1 --host 0.0.0.0 --port 8000",
    })
    ```

    You can run `envd exec --command train` to train the model.
    """


def expose(
    envd_port: int,
    host_port: Optional[int],
    service: Optional[str],
    listen_addr: Optional[str],
):
    """Expose port to host
    Proposal: https://github.com/tensorchord/envd/pull/780

    Args:
        envd_port (int): port in `envd` container
        host_port (Optional[int]): port in the host, if not provided or
            `host_port=0`, `envd` will randomly choose a free port
        service (Optional[str]): service name
        listen_addr (Optional[str]): address to listen on
    """


def daemon(commands: List[List[str]]):
    """Run daemon processes in the container
    Proposal: https://github.com/tensorchord/envd/pull/769

    It's better to redirect the logs to local files for debugging purposes.

    You can find the generated horust config files under `/etc/horust/services`
    and log files under `/var/log/horust` in the container.

    Args:
        commands (List[List[str]]): run multiple commands in the background

    Example usage:
    ```python
    runtime.daemon(commands=[
        ["jupyter-lab", "--port", "8080"],
        ["python3", "serving.py", ">>serving.log", "2>&1"],
    ])
    ```
    """


def environ(env: Dict[str, str], extra_path: List[str]):
    """Add runtime environments

    Args:
        env (Dict[str, str]): environment name to value
        extra_path (List[str]): additional PATH

    Example usage:
    ```python
    runtime.environ(env={"ENVD_MODE": "DEV"}, extra_path=["/usr/bin/go/bin"])
    ```
    """


def mount(host_path: str, envd_path: str):
    """Mount from host path to container path (runtime)

    Args:
        host_path (str): source path in the host machine
        envd_path (str): destination path in the envd container
    """


def init(commands: List[str]):
    """Commands to be executed when start the container

    You can find the generated horust config files under `/etc/horust/services`
    and log files under `/var/log/horust` in the container.

    Args:
        commands (List[str]): list of commands
    """



# Copyright 2022 The envd Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Global functions

::: tip
Note that the documentation is automatically generated from [envd/api](https://github.com/tensorchord/envd/tree/main/envd/api) folder
in [tensorchord/envd](https://github.com/tensorchord/envd/tree/main/envd/api) repo.
Please update the python file there instead of directly editing file inside envd-docs repo.
:::
"""

from typing import List, Optional


def base(os: str, language: str, image: Optional[str]):
    """Set base image

    Args:
        os (str): The operating system (i.e. `ubuntu20.04`)
        language (str): The programming language dependency (i.e. `python3.8`)
        image (Optional[str]): Custom image (i.e. `python:3.9-slim`)
    """


def shell(name: str):
    """Interactive shell

    Args:
        name (str): shell name (i.e. `zsh`, `bash`)
    """


def run(commands: List[str], mount_host: bool = False):
    """Execute command

    Args:
        commands (List[str]): command to run during the building process
        mount_host (bool): mount the host directory. Default is False.
            Enabling this will disable the build cache for this operation.

    Example:
    ```python
    run(commands=["conda install -y -c conda-forge exa"])
    ```
    """


def git_config(
    name: Optional[str] = None,
    email: Optional[str] = None,
    editor: Optional[str] = None,
):
    """Setup git config

    Args:
        name (optional, str): User name
        email (optional, str): User email
        editor (optional, str): Editor for git operations

    Example usage:
    ```python
    git_config(name="My Name", email="my@email.com", editor="vim")
    ```
    """


def include(git: str):
    """Import from another git repo

    This will pull the git repo and execute all the `envd` files. The return value will be a module
    contains all the variables/functions defined (except the ones with `_` prefix).

    Args:
        git (str): git URL

    Example usage:
    ```python
    envd = include("https://github.com/tensorchord/envdlib")

    def build():
        base(os="ubuntu20.04", language="python")
        envd.tensorboard(8000)
    ```
    """



# Copyright 2022 The envd Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from setuptools.command.sdist import sdist
from setuptools import setup, Extension, find_packages
from setuptools.command.build_ext import build_ext
from wheel.bdist_wheel import bdist_wheel


import subprocess
import logging

with open("README.md", "r", encoding="utf-8") as f:
    readme = f.read()


def build_envd_if_not_found():
    if not os.path.isfile("bin/envd"):
        logging.info("envd not found. Build from scratch")
        try:
            with open(".GIT_TAG_INFO") as f:
                logging.info("Use build_tag from .GIT_TAG_INFO")
                tag = f.read().strip()
                logging.info("start building envd from source")
                errno = subprocess.call(
                    ["make", "build-release", "GIT_TAG={}".format(tag)]
                )
        except OSError:
            logging.warning(".GIT_TAG_INFO not found")
            logging.info("start building envd from source")
            errno = subprocess.call(["make", "build-release"])
        assert errno == 0, "Failed to build envd"


class EnvdExtension(Extension):
    """Extension for `envd`"""


class EnvdBuildExt(build_ext):
    def build_extension(self, ext: Extension) -> None:
        if not isinstance(ext, EnvdExtension):
            return super().build_extension(ext)

        build_envd_if_not_found()


class bdist_wheel_universal(bdist_wheel):
    def get_tag(self):
        *_, plat = super().get_tag()
        return "py2.py3", "none", plat


class SdistCommand(sdist):
    def run(self):
        errno = subprocess.call(["make", "generate-git-tag-info"])
        assert errno == 0, "Failed to generate git tag info"
        if not os.path.isfile(".GIT_TAG_INFO"):
            logging.warning(".GIT_TAG_INFO not found")
        else:
            logging.info(".GIT_TAG_INFO generated")
        super().run()


def get_version():
    # Remove prefix v in versioning
    build_envd_if_not_found()
    version = subprocess.check_output(
        ["./bin/envd", "version", "--short"], universal_newlines=True
    ).strip()
    ver = version.rsplit(" ", 1)[-1][1:]
    return ver


classifiers = [
    "Development Status :: 3 - Alpha",
    "Topic :: Software Development :: Build Tools",
    "Intended Audience :: Science/Research",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: Apache Software License",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Programming Language :: Python :: 3.11",
]


setup(
    name="envd",
    version=get_version(),
    use_scm_version=True,
    setup_requires=["setuptools_scm"],
    description="A development environment management tool for data scientists.",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/tensorchord/envd",
    license="Apache License 2.0",
    author="TensorChord",
    author_email="envd-maintainers@tensorchord.ai",
    packages=find_packages(),
    include_package_data=True,
    python_requires=">=3.6",
    data_files=[("bin", ["bin/envd"])],
    classifiers=classifiers,
    zip_safe=False,
    ext_modules=[
        EnvdExtension(name="envd", sources=["cmd/*"]),
    ],
    cmdclass=dict(
        build_ext=EnvdBuildExt,
        sdist=SdistCommand,
        bdist_wheel=bdist_wheel_universal,
    ),
)
